import React, { useCallback } from "react";
import { Typography, makeStyles, Link as MuiLink } from "@material-ui/core";
import { SimpleAccordion } from "../common/Accordion/SimpleAccordion";
import { IOCDetails } from "./IOCDetails";
import { Link as ReactRouterLink } from "react-router-dom";
import { LokiContainer } from "../common/Loki/LokiContainer";
import { RecordSearch } from "../records/RecordSearch";
import GitRefLink from "./GitRefLink";
import AlertMessages from "./AlertMessages";
import AccessControl from "../../components/auth/AccessControl";
import { IocActiveDeployment } from "../../api/DataTypes";
import { theme } from "../../style/Theme";
import useUrlState from "@ahooksjs/use-url-state";
import {
  serialize,
  deserialize
} from "../../components/common/URLState/URLState";

const useStyles = makeStyles((theme) => ({
  iocNotDeployed: {
    color: theme.iocNotDeployed.color
  }
}));

export function IOCLiveStatus({ ioc }) {
  const classes = useStyles(theme);
  const [state, setState] = useUrlState({
    procserv_log_open: "true",
    records_open: "false"
  });

  const liveIOC = { ...ioc };
  liveIOC.name = ioc.namingName;

  const getSubset = useCallback(
    (ioc) => {
      const iocHasDeployment = new IocActiveDeployment(
        ioc.activeDeployment
      ).hasDeployment();

      // if IOC is not deployed some fields has to show '---' value
      const grayOutNoText = {
        shownValue: "---",
        shownClass: classes.iocNotDeployed
      };

      let subset = {
        Description: ioc.description,
        Revision: iocHasDeployment ? (
          <GitRefLink deployment={ioc.activeDeployment} />
        ) : (
          grayOutNoText
        ),
        "Deployment comment": grayOutNoText,
        "Deployed on":
          iocHasDeployment && ioc.activeDeployment?.host.csEntryIdValid ? (
            <Typography>
              {ioc.activeDeployment?.host.csEntryId ? (
                <MuiLink
                  component={ReactRouterLink}
                  to={`/hosts/${ioc.activeDeployment?.host.csEntryId}`}
                >
                  {ioc.activeDeployment?.host.fqdn}
                </MuiLink>
              ) : (
                <>{ioc.activeDeployment?.host.fqdn}</>
              )}
            </Typography>
          ) : (
            grayOutNoText
          ),
        Maintainer: ioc.owner
      };

      // show comment only for deployments and where comment field is filled
      if (ioc.activeDeployment && !ioc.activeDeployment.isUndeployment) {
        subset["Deployment comment"] =
          ioc.activeDeployment?.comment?.length > 0
            ? ioc.activeDeployment?.comment
            : grayOutNoText;
      }

      return subset;
    },
    [classes.iocNotDeployed]
  );

  function isIocDeployed(ioc) {
    return Boolean(ioc.activeDeployment);
  }

  return (
    <>
      <IOCDetails
        ioc={liveIOC}
        getSubset={getSubset}
        alert={<AlertMessages alerts={liveIOC.alerts} />}
      />
      {liveIOC.activeDeployment?.host?.csEntryIdValid && (
        <AccessControl
          allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
          renderNoAccess={() => <></>}
        >
          <SimpleAccordion
            summary="ProcServLog info"
            expanded={deserialize(state.procserv_log_open)}
            onChange={(event, expanded) =>
              setState({ procserv_log_open: serialize(expanded) })
            }
          >
            <LokiContainer
              csEntryId={liveIOC.activeDeployment?.host.csEntryId}
              iocName={ioc.namingName}
              isDeployed={isIocDeployed(ioc)}
            />
          </SimpleAccordion>
          <SimpleAccordion
            summary="Records"
            expanded={deserialize(state.records_open)}
            onChange={(event, expanded) =>
              setState({ records_open: serialize(expanded) })
            }
          >
            <RecordSearch
              iocName={ioc.namingName}
              rowType="iocDetails"
            />
          </SimpleAccordion>
        </AccessControl>
      )}
    </>
  );
}
