import {
  Button,
  Link as MuiLink,
  Tooltip,
  Typography
} from "@material-ui/core";
import React, { useState, useEffect, useContext, useCallback } from "react";
import { IOCDetails } from "../../components/IOC/IOCDetails";
import { DeployIOC } from "../../components/IOC/DeployIOC";
import { UndeployIOC } from "../../components/IOC/UndeployIOC";
import { SimpleModal } from "../../components/common/SimpleModal/SimpleModal";
import {
  useUpdateAndDeployIoc,
  useCreateUndeployment,
  useDeploymentListForIOC,
  useDeploymentJobById,
  userContext
} from "../../api/SwaggerApi";
import AlertMessages from "./AlertMessages";
import { SimpleAccordion } from "../common/Accordion/SimpleAccordion";
import { initRequestParams } from "../common/Helper";
import AccessControl from "../auth/AccessControl";
import { JobAsyncList } from "../Job/JobAsyncList";
import { DeploymentStatus } from "../../api/DataTypes";
import { IOCService } from "./IOCService";
import useUrlState from "@ahooksjs/use-url-state";
import {
  serialize,
  deserialize
} from "../../components/common/URLState/URLState";

export function IOCManage({
  ioc,
  getIOC,
  buttonDisabled,
  currentCommand,
  commands,
  commandsLoading,
  getCommands,
  setButtonDisabled,
  commandColumnSort,
  setCommandColumnSort,
  commandLazyParams,
  setCommandLazyParams
}) {
  const { user } = useContext(userContext);
  const [state, setState] = useUrlState({
    control_log_open: "false",
    deployment_log_open: "false",
    deployments_rows: "5",
    deployments_page: "0"
  });

  const [deployDialogOpen, setDeployDialogOpen] = useState(false);
  const [undeployDialogOpen, setUndeployDialogOpen] = useState(false);
  const [deployments, getDeployments] = useDeploymentListForIOC();
  const [deploymentList, setDeploymentList] = useState([]);
  const [deploymentJob, getDeploymentJobById] = useDeploymentJobById();
  const [asyncCommands, setAsyncCommands] = useState(commands.operationsList);

  const [columnSort, setColumnSort] = useState({
    sortField: null,
    sortOrder: null
  });

  const lazyParams = useCallback(() => {
    return {
      rows: deserialize(state.deployments_rows),
      page: deserialize(state.deployments_page)
    };
  }, [state]);

  const setLazyParams = useCallback(
    (params) => {
      setState({
        deployments_rows: serialize(params.rows),
        deployments_page: serialize(params.page)
      });
    },
    [setState]
  );

  const rowsPerPage = [5, 20];

  useEffect(() => {
    if (ioc.activeDeployment?.awxJobId) {
      getDeploymentJobById(ioc.activeDeployment?.awxJobId);
    }
  }, [getDeploymentJobById, ioc.activeDeployment?.awxJobId]);

  useEffect(() => {
    setAsyncCommands(commands.operationsList);
  }, [commands.operationsList]);

  useEffect(() => {
    setDeploymentList(deployments.operationsList);
  }, [deployments, setDeploymentList]);

  useEffect(() => {
    if (ioc) {
      let requestParams = initRequestParams(lazyParams(), null, columnSort);

      requestParams.ioc_id = ioc.id;

      if (columnSort.sortField === "start") {
        requestParams.order_by = "TIME";
      }

      if (columnSort.sortField === "user") {
        requestParams.order_by = "CREATED_BY";
      }

      if (columnSort.sortField === "ioc") {
        requestParams.order_by = "IOC_NAME";
      }

      if (columnSort.sortField === "version") {
        requestParams.order_by = "GIT_REFERENCE";
      }

      getDeployments(requestParams);
    }
  }, [ioc, lazyParams, columnSort, getDeployments]);

  const closeDeployModal = () => {
    setDeployDialogOpen(false);
    getIOC();
    // history.push(window.location.pathname);
  };

  const closeUndeployModal = () => {
    setUndeployDialogOpen(false);
  };

  const getSubset = useCallback(
    (ioc) => {
      ioc = { ...ioc };
      const { sourceUrl: git } = ioc;

      // Show START/STOP components only when IOC was deployed SUCCESSFULLY
      const deploymentStatus = new DeploymentStatus(
        ioc.activeDeployment,
        deploymentJob
      );
      const showControls = deploymentStatus.wasSuccessful();

      let subset = {
        "IOC name": (
          <Typography>
            <MuiLink
              href={`${window.CCDB_ADDRESS}?name=${ioc.namingName}`}
              target="_blank"
              rel="noreferrer"
            >
              {ioc.namingName}
            </MuiLink>
          </Typography>
        ),
        git: (
          <Typography>
            <MuiLink
              href={git}
              target="_blank"
              rel="noreferrer"
            >
              {git}
            </MuiLink>
          </Typography>
        )
      };

      if (user) {
        subset["IOC Service Controls"] = showControls ? (
          <IOCService
            {...{
              ioc,
              currentCommand,
              getCommands,
              buttonDisabled,
              setButtonDisabled,
              commandColumnSort,
              commandLazyParams
            }}
          />
        ) : (
          "Can't be controlled, because IOC is not (yet) deployed succesfully"
        );
      }

      return subset;
    },
    [
      buttonDisabled,
      commandColumnSort,
      commandLazyParams,
      currentCommand,
      deploymentJob,
      getCommands,
      setButtonDisabled,
      user
    ]
  );

  let content = <></>;
  if (ioc) {
    console.log({ ...ioc });
    const managedIOC = { ...ioc };

    const formInit = {
      name: ioc.namingName,
      description: ioc.description,
      version: ioc.sourceVersion,
      shortVersion: ioc.sourceVersionShort,
      git: ioc.sourceUrl,
      gitProjectId: ioc.gitProjectId
    };

    if (ioc.activeDeployment) {
      formInit.csEntryHost = {
        fqdn: ioc.activeDeployment.host.fqdn,
        id: ioc.activeDeployment.host.csEntryId
      };
    }

    let disabledDebployButtonTitle = "";
    if (buttonDisabled || ioc.operationInProgress) {
      disabledDebployButtonTitle =
        "There is an ongoing operation, you can not deploy the IOC right now";
    }

    let disabledUndeployButtonTitle = "";
    if (buttonDisabled || ioc.operationInProgress) {
      disabledUndeployButtonTitle =
        "There is an ongoing operation, you can not undeploy the IOC right now";
    }

    content = (
      <>
        <IOCDetails
          ioc={managedIOC}
          getSubset={getSubset}
          alert={<AlertMessages alerts={managedIOC.alerts} />}
          buttons={
            <>
              <Tooltip title={disabledDebployButtonTitle}>
                <span>
                  <Button
                    variant="contained"
                    color="secondary"
                    onClick={() => {
                      setDeployDialogOpen(true);
                    }}
                    disabled={buttonDisabled || ioc.operationInProgress}
                  >
                    {ioc.activeDeployment ? "Deploy new revision" : "Deploy"}
                  </Button>
                </span>
              </Tooltip>
              {ioc.activeDeployment ? (
                <Tooltip title={disabledUndeployButtonTitle}>
                  <span>
                    <Button
                      variant="contained"
                      color="secondary"
                      style={{ marginRight: 20 }}
                      onClick={() => {
                        setUndeployDialogOpen(true);
                      }}
                      disabled={buttonDisabled || ioc.operationInProgress}
                    >
                      Undeploy
                    </Button>
                  </span>
                </Tooltip>
              ) : (
                <></>
              )}
            </>
          }
        />
        <AccessControl
          allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
          renderNoAccess={() => <></>}
        >
          <SimpleAccordion
            summary="IOC Service Control log"
            expanded={deserialize(state.control_log_open)}
            onChange={(event, expanded) =>
              setState({ control_log_open: serialize(expanded) })
            }
          >
            <JobAsyncList
              jobs={asyncCommands}
              rowType="iocServiceControl"
              setJobs={setAsyncCommands}
              loading={commandsLoading}
              totalCount={commands.totalCount}
              lazyParams={commandLazyParams}
              setLazyParams={setCommandLazyParams}
              columnSort={commandColumnSort}
              setColumnSort={setCommandColumnSort}
              rowsPerPage={rowsPerPage}
              deploymentId={ioc?.activeDeployment?.id}
            />
          </SimpleAccordion>
          <SimpleAccordion
            summary="Deployment log"
            expanded={deserialize(state.deployment_log_open)}
            onChange={(event, expanded) =>
              setState({ deployment_log_open: serialize(expanded) })
            }
          >
            <JobAsyncList
              jobs={deploymentList}
              setJobs={setDeploymentList}
              totalCount={deployments.totalCount}
              lazyParams={lazyParams()}
              setLazyParams={setLazyParams}
              columnSort={columnSort}
              setColumnSort={setColumnSort}
              rowsPerPage={rowsPerPage}
            />
          </SimpleAccordion>
          <SimpleModal
            open={deployDialogOpen}
            setOpen={setDeployDialogOpen}
          >
            <DeployIOC
              open={deployDialogOpen}
              setOpen={setDeployDialogOpen}
              submitCallback={closeDeployModal}
              init={formInit}
              hook={useUpdateAndDeployIoc.bind(null, ioc.id)}
              hasActiveDeployment={ioc.activeDeployment}
            />
          </SimpleModal>
          <SimpleModal
            open={undeployDialogOpen}
            setOpen={setUndeployDialogOpen}
          >
            <UndeployIOC
              open={undeployDialogOpen}
              setOpen={setUndeployDialogOpen}
              submitCallback={closeUndeployModal}
              ioc={ioc}
              hook={useCreateUndeployment.bind(null, ioc.id)}
            />
          </SimpleModal>
        </AccessControl>
      </>
    );
  }

  return <>{content}</>;
}
